package kr.co.vividnext.sodalive.admin.audition

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.audition.Audition
import kr.co.vividnext.sodalive.audition.QAudition.audition
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface AdminAuditionRepository : JpaRepository<Audition, Long>, AdminAuditionQueryRepository

interface AdminAuditionQueryRepository {
    fun getAuditionList(offset: Long, limit: Long): List<GetAuditionListItem>
    fun getAuditionListCount(): Int
    fun getAuditionDetail(auditionId: Long): GetAuditionDetailRawData
}

class AdminAuditionQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) : AdminAuditionQueryRepository {
    override fun getAuditionList(offset: Long, limit: Long): List<GetAuditionListItem> {
        return queryFactory
            .select(
                QGetAuditionListItem(
                    audition.id,
                    audition.title,
                    audition.imagePath.prepend("/").prepend(coverImageHost),
                    audition.isAdult,
                    audition.information,
                    audition.status,
                    audition.originalWorkUrl.coalesce("")
                )
            )
            .from(audition)
            .where(audition.isActive.isTrue)
            .offset(offset)
            .limit(limit)
            .orderBy(audition.isActive.desc(), audition.id.desc())
            .fetch()
    }

    override fun getAuditionListCount(): Int {
        return queryFactory
            .select(audition.id)
            .from(audition)
            .fetch()
            .size
    }

    override fun getAuditionDetail(auditionId: Long): GetAuditionDetailRawData {
        return queryFactory
            .select(
                QGetAuditionDetailRawData(
                    audition.id,
                    audition.title,
                    audition.imagePath.prepend("/").prepend(coverImageHost),
                    audition.information,
                    audition.originalWorkUrl.coalesce("")
                )
            )
            .from(audition)
            .where(audition.id.eq(auditionId))
            .fetchFirst()
    }
}
