package kr.co.vividnext.sodalive.admin.content

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/audio-content")
class AdminContentController(private val service: AdminContentService) {
    @GetMapping("/list")
    fun getAudioContentList(
        @RequestParam(value = "status", required = false) status: ContentReleaseStatus?,
        pageable: Pageable
    ) = ApiResponse.ok(
        service.getAudioContentList(
            status = status ?: ContentReleaseStatus.OPEN,
            pageable
        )
    )

    @GetMapping("/search")
    fun searchAudioContent(
        @RequestParam(value = "status", required = false) status: ContentReleaseStatus?,
        @RequestParam(value = "search_word") searchWord: String,
        pageable: Pageable
    ) = ApiResponse.ok(
        service.searchAudioContent(
            status = status ?: ContentReleaseStatus.OPEN,
            searchWord,
            pageable
        )
    )

    @PutMapping
    fun modifyAudioContent(
        @RequestBody request: UpdateAdminContentRequest
    ) = ApiResponse.ok(service.updateAudioContent(request))

    @GetMapping("/main/tab")
    fun getContentMainTabList() = ApiResponse.ok(service.getContentMainTabList())
}

enum class ContentReleaseStatus {
    // 콘텐츠가 공개된 상태
    OPEN,

    // 예약된 콘텐츠, 아직 공개되지 않은 상태
    SCHEDULED
}
