package kr.co.vividnext.sodalive.admin.content.curation.tag

import kr.co.vividnext.sodalive.admin.content.curation.AddItemToCurationRequest
import kr.co.vividnext.sodalive.admin.content.curation.RemoveItemInCurationRequest
import kr.co.vividnext.sodalive.admin.content.curation.SearchCurationItemResponse
import kr.co.vividnext.sodalive.admin.content.curation.UpdateCurationItemOrdersRequest
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.main.curation.tag.ContentHashTagCuration
import kr.co.vividnext.sodalive.content.main.curation.tag.ContentHashTagCurationItem
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class AdminHashTagCurationService(
    private val repository: AdminHashTagCurationRepository,
    private val itemRepository: AdminContentHashTagCurationItemRepository,
    private val audioContentRepository: AudioContentRepository
) {
    @Transactional
    fun createContentHashTagCuration(request: CreateContentHashTagCurationRequest) {
        var tag = request.tag.trim()
        if (!tag.startsWith("#")) {
            tag = "#$tag"
        }

        val isExists = repository.isExistsTag(tag = tag)

        if (isExists) {
            throw SodaException("이미 등록된 태그 입니다.")
        }

        repository.save(
            ContentHashTagCuration(
                tag = tag,
                isAdult = request.isAdult
            )
        )
    }

    @Transactional
    fun updateContentHashTagCuration(request: UpdateContentHashTagCurationRequest) {
        val hashTagCuration = repository.findByIdOrNull(id = request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (request.tag != null) {
            var tag = request.tag.trim()
            if (!tag.startsWith("#")) {
                tag = "#$tag"
            }

            hashTagCuration.tag = tag
        }

        if (request.isAdult != null) {
            hashTagCuration.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            hashTagCuration.isActive = request.isActive
        }
    }

    @Transactional
    fun updateContentHashTagCurationOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val contentHashTagCuration = repository.findByIdOrNull(ids[index])

            if (contentHashTagCuration != null) {
                contentHashTagCuration.orders = index + 1
            }
        }
    }

    fun getContentHashTagCurationList(): List<GetAdminContentHashTagCurationResponse> {
        return repository.getContentHashTagCurationList()
    }

    fun getHashTagCurationItemList(curationId: Long): List<GetAdminHashTagCurationItemResponse> {
        return itemRepository.getContentHashTagCurationItemList(curationId)
    }

    fun searchHashTagCurationContentItem(curationId: Long, searchWord: String): List<SearchCurationItemResponse> {
        return repository.searchHashTagCurationContentItem(curationId, searchWord)
    }

    @Transactional
    fun addItemToHashTagCuration(request: AddItemToCurationRequest) {
        val curation = repository.findByIdOrNull(id = request.curationId)
            ?: throw SodaException("잘못된 요청입니다.")

        request.itemIdList.forEach { contentId ->
            val audioContent = audioContentRepository.findByIdAndActive(contentId)

            if (audioContent != null) {
                val item = itemRepository.findByCurationIdAndContentId(
                    curationId = request.curationId,
                    contentId = audioContent.id
                ) ?: ContentHashTagCurationItem()
                item.curation = curation
                item.content = audioContent
                item.isActive = true
                itemRepository.save(item)
            }
        }
    }

    @Transactional
    fun removeItemInHashTagCuration(request: RemoveItemInCurationRequest) {
        val item = itemRepository.findByCurationIdAndItemId(
            curationId = request.curationId,
            itemId = request.itemId
        )

        item?.isActive = false
    }

    @Transactional
    fun updateItemInHashTagCurationOrders(request: UpdateCurationItemOrdersRequest) {
        val ids = request.itemIds
        for (index in ids.indices) {
            val item = itemRepository.findByCurationIdAndItemId(
                curationId = request.curationId,
                itemId = ids[index]
            )

            if (item != null) {
                item.orders = index + 1
            }
        }
    }
}
