package kr.co.vividnext.sodalive.admin.content.series.recommend

import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.admin.content.series.AdminContentSeriesRepository
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.main.tab.RecommendSeries
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class AdminRecommendSeriesService(
    private val s3Uploader: S3Uploader,
    private val repository: AdminRecommendSeriesRepository,
    private val seriesRepository: AdminContentSeriesRepository,
    private val objectMapper: ObjectMapper,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String
) {
    fun getRecommendSeriesList(isFree: Boolean): List<GetAdminRecommendSeriesListResponse> {
        return repository.getRecommendSeriesList(isFree = isFree)
    }

    @Transactional
    fun createRecommendSeries(image: MultipartFile, requestString: String) {
        val request = objectMapper.readValue(requestString, CreateRecommendSeriesRequest::class.java)
        val series = seriesRepository.findByIdOrNull(request.seriesId)
            ?: throw SodaException("잘못된 요청입니다.")

        val recommendSeries = RecommendSeries(isFree = request.isFree)
        recommendSeries.series = series
        repository.save(recommendSeries)

        val fileName = generateFileName()
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "recommend_series/${recommendSeries.id}/$fileName"
        )
        recommendSeries.imagePath = imagePath
    }

    @Transactional
    fun updateRecommendSeries(image: MultipartFile?, requestString: String) {
        val request = objectMapper.readValue(requestString, UpdateRecommendSeriesRequest::class.java)
        val recommendSeries = repository.findByIdOrNull(request.id)
            ?: throw SodaException("잘못된 요청입니다.")

        if (image != null) {
            val fileName = generateFileName()
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "recommend_series/${recommendSeries.id}/$fileName"
            )
            recommendSeries.imagePath = imagePath
        }

        if (request.isActive != null) {
            recommendSeries.isActive = request.isActive
        }

        if (request.seriesId != null) {
            val series = seriesRepository.findByIdOrNull(request.seriesId)

            if (series != null) {
                recommendSeries.series = series
            }
        }
    }

    @Transactional
    fun updateRecommendSeriesOrders(ids: List<Long>) {
        for (index in ids.indices) {
            val recommendSeries = repository.findByIdOrNull(ids[index])

            if (recommendSeries != null) {
                recommendSeries.orders = index + 1
            }
        }
    }
}
