package kr.co.vividnext.sodalive.admin.event.charge

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
@Transactional(readOnly = true)
class AdminChargeEventService(private val repository: AdminChargeEventRepository) {
    @Transactional
    fun createChargeEvent(request: CreateChargeEventRequest): Long {
        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
        val startDate = LocalDateTime.parse(request.startDateString, dateTimeFormatter)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val endDate = LocalDateTime.parse(request.endDateString, dateTimeFormatter).withSecond(59)
            .atZone(ZoneId.of("Asia/Seoul"))
            .withZoneSameInstant(ZoneId.of("UTC"))
            .toLocalDateTime()

        val chargeEvent = ChargeEvent(
            title = request.title,
            startDate = startDate,
            endDate = endDate,
            availableCount = request.availableCount,
            addPercent = request.addPercent / 100f
        )

        return repository.save(chargeEvent).id!!
    }

    @Transactional
    fun modifyChargeEvent(request: ModifyChargeEventRequest) {
        val chargeEvent = repository.findByIdOrNull(request.id)
            ?: throw SodaException("해당하는 충전이벤트가 없습니다\n다시 시도해 주세요.")

        if (request.title != null) {
            chargeEvent.title = request.title
        }

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")
        if (request.startDateString != null) {
            chargeEvent.startDate = LocalDateTime.parse(request.startDateString, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.endDateString != null) {
            chargeEvent.endDate = LocalDateTime.parse(request.endDateString, dateTimeFormatter).withSecond(59)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.availableCount != null) {
            chargeEvent.availableCount = request.availableCount
        }

        if (request.addPercent != null) {
            chargeEvent.addPercent = request.addPercent / 100f
        }

        if (request.isActive != null) {
            chargeEvent.isActive = request.isActive
        }
    }

    fun getChargeEventList(): List<GetChargeEventListResponse> {
        return repository.getChargeEventList()
    }
}
