package kr.co.vividnext.sodalive.admin.live.signature

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.signature.SignatureCanSortType
import org.springframework.data.domain.Pageable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('ADMIN')")
@RequestMapping("/admin/live/signature-can")
class AdminSignatureCanController(private val service: AdminSignatureCanService) {
    @GetMapping
    fun getSignatureCanList(
        pageable: Pageable,
        @RequestParam("sort-type", required = false) sortType: SignatureCanSortType?
    ) = ApiResponse.ok(data = service.getSignatureCanList(pageable, sortType ?: SignatureCanSortType.NEWEST))

    @PostMapping
    fun createSignatureCan(
        @RequestParam("can") can: Int,
        @RequestParam("time") time: Int,
        @RequestParam("image") image: MultipartFile,
        @RequestParam("creator_id") creatorId: Long,
        @RequestParam("isAdult", required = false) isAdult: Boolean = false
    ) = ApiResponse.ok(
        service.createSignatureCan(can = can, time = time, creatorId = creatorId, image = image, isAdult = isAdult),
        "등록되었습니다."
    )

    @PutMapping
    fun modifySignatureCan(
        @RequestParam("id") id: Long,
        @RequestParam("can", required = false) can: Int?,
        @RequestParam("time", required = false) time: Int?,
        @RequestParam("image", required = false) image: MultipartFile?,
        @RequestParam("isActive", required = false) isActive: Boolean?,
        @RequestParam("isAdult", required = false) isAdult: Boolean?
    ) = run {
        if (can == null && time == null && image == null && isActive == null && isAdult == null) {
            throw SodaException("변경사항이 없습니다.")
        }

        ApiResponse.ok(
            service.modifySignatureCan(
                id = id,
                can = can,
                time = time,
                image = image,
                isActive = isActive,
                isAdult = isAdult
            ),
            "수정되었습니다."
        )
    }
}
