package kr.co.vividnext.sodalive.admin.live.signature

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.signature.QSignatureCan.signatureCan
import kr.co.vividnext.sodalive.live.signature.SignatureCan
import kr.co.vividnext.sodalive.live.signature.SignatureCanSortType
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository

interface AdminSignatureCanRepository : JpaRepository<SignatureCan, Long>, AdminSignatureCanQueryRepository

interface AdminSignatureCanQueryRepository {
    fun getSignatureCanListTotalCount(): Int

    fun getSignatureCanList(
        sortType: SignatureCanSortType,
        imageHost: String,
        offset: Long,
        limit: Long
    ): List<GetSignatureCanListItem>
}

class AdminSignatureCanQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AdminSignatureCanQueryRepository {
    override fun getSignatureCanListTotalCount(): Int {
        return queryFactory.select(signatureCan.id)
            .from(signatureCan)
            .where(signatureCan.isActive.isTrue)
            .fetch()
            .size
    }

    override fun getSignatureCanList(
        sortType: SignatureCanSortType,
        imageHost: String,
        offset: Long,
        limit: Long
    ): List<GetSignatureCanListItem> {
        val orderBy = when (sortType) {
            SignatureCanSortType.CAN_HIGH -> {
                signatureCan.can.desc()
            }

            SignatureCanSortType.CAN_LOW -> {
                signatureCan.can.asc()
            }

            else -> {
                signatureCan.id.desc()
            }
        }

        return queryFactory.select(
            QGetSignatureCanListItem(
                signatureCan.id,
                signatureCan.can,
                signatureCan.time,
                signatureCan.image.prepend("/").prepend(imageHost),
                signatureCan.isAdult,
                member.nickname
            )
        )
            .from(signatureCan)
            .innerJoin(signatureCan.creator, member)
            .where(signatureCan.isActive.isTrue)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc(), orderBy)
            .fetch()
    }
}
