package kr.co.vividnext.sodalive.admin.point

import kr.co.vividnext.sodalive.common.SodaException
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class PointPolicyService(private val repository: PointPolicyRepository) {
    fun getAll(offset: Long, limit: Long): GetPointRewardPolicyListResponse {
        val totalCount = repository.getTotalCount()
        val items = repository.getAll(offset, limit)

        return GetPointRewardPolicyListResponse(totalCount, items)
    }

    @Transactional
    fun create(request: CreatePointRewardPolicyRequest) {
        val pointPolicy = request.toEntity()
        repository.save(pointPolicy)
    }

    @Transactional
    fun update(id: Long, request: ModifyPointRewardPolicyRequest) {
        val pointPolicy = repository.findByIdOrNull(id)
            ?: throw SodaException("잘못된 접근입니다.")

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm")

        if (request.title != null) {
            pointPolicy.title = request.title
        }

        if (request.startDate != null) {
            pointPolicy.startDate = LocalDateTime.parse(request.startDate, dateTimeFormatter)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.endDate != null) {
            pointPolicy.endDate = LocalDateTime.parse(request.endDate, dateTimeFormatter).withSecond(59)
                .atZone(ZoneId.of("Asia/Seoul"))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        }

        if (request.isActive != null) {
            pointPolicy.isActive = request.isActive
        }
    }
}
