package kr.co.vividnext.sodalive.agora

import org.apache.commons.codec.binary.Base64
import java.security.InvalidKeyException
import java.security.NoSuchAlgorithmException
import java.security.SecureRandom
import java.util.Date
import java.util.zip.CRC32
import javax.crypto.Mac
import javax.crypto.spec.SecretKeySpec

object AgoraUtils {
    const val HMAC_SHA256_LENGTH: Long = 32
    const val VERSION_LENGTH = 3
    const val APP_ID_LENGTH = 32

    @Throws(InvalidKeyException::class, NoSuchAlgorithmException::class)
    fun hmacSign(keyString: String, msg: ByteArray?): ByteArray {
        val keySpec = SecretKeySpec(keyString.toByteArray(), "HmacSHA256")
        val mac = Mac.getInstance("HmacSHA256")
        mac.init(keySpec)
        return mac.doFinal(msg)
    }

    fun pack(packableEx: PackableEx): ByteArray {
        val buffer = ByteBuf()
        packableEx.marshal(buffer)
        return buffer.asBytes()
    }

    fun unpack(data: ByteArray?, packableEx: PackableEx) {
        val buffer = ByteBuf(data!!)
        packableEx.unmarshal(buffer)
    }

    fun base64Encode(data: ByteArray?): String {
        val encodedBytes: ByteArray = Base64.encodeBase64(data)
        return String(encodedBytes)
    }

    fun base64Decode(data: String): ByteArray {
        return Base64.decodeBase64(data.toByteArray())
    }

    fun crc32(data: String): Int {
        // get bytes from string
        val bytes = data.toByteArray()
        return crc32(bytes)
    }

    fun crc32(bytes: ByteArray?): Int {
        val checksum = CRC32()
        checksum.update(bytes)
        return checksum.value.toInt()
    }

    fun getTimestamp(): Int {
        return (Date().time / 1000).toInt()
    }

    fun randomInt(): Int {
        return SecureRandom().nextInt()
    }

    fun isUUID(uuid: String): Boolean {
        return if (uuid.length != 32) {
            false
        } else {
            uuid.matches("\\p{XDigit}+".toRegex())
        }
    }
}
