package kr.co.vividnext.sodalive.content

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.category.QCategoryContent.categoryContent
import kr.co.vividnext.sodalive.content.comment.QAudioContentComment.audioContentComment
import kr.co.vividnext.sodalive.content.like.QAudioContentLike.audioContentLike
import kr.co.vividnext.sodalive.content.main.ContentCreatorResponse
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentMainItem
import kr.co.vividnext.sodalive.content.main.QGetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBanner
import kr.co.vividnext.sodalive.content.main.banner.QAudioContentBanner.audioContentBanner
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCurationItem.audioContentCurationItem
import kr.co.vividnext.sodalive.content.main.tab.QAudioContentMainTab.audioContentMainTab
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.content.pin.QPinContent.pinContent
import kr.co.vividnext.sodalive.content.playlist.AudioContentPlaylistContent
import kr.co.vividnext.sodalive.content.playlist.QAudioContentPlaylistContent
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.event.QEvent.event
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AudioContentRepository : JpaRepository<AudioContent, Long>, AudioContentQueryRepository

interface AudioContentQueryRepository {
    fun findByIdAndActive(contentId: Long): AudioContent?
    fun findByIdAndCreatorId(contentId: Long, creatorId: Long): AudioContent?
    fun findByCreatorId(
        creatorId: Long,
        coverImageHost: String,
        isAdult: Boolean = false,
        contentType: ContentType = ContentType.ALL,
        sortType: SortType = SortType.NEWEST,
        categoryId: Long = 0,
        offset: Long = 0,
        limit: Long = 10
    ): List<GetAudioContentListItem>

    fun findTotalCountByCreatorId(
        creatorId: Long,
        isAdult: Boolean = false,
        categoryId: Long = 0,
        contentType: ContentType = ContentType.ALL
    ): Int

    fun getCreatorOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        creatorId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse>

    fun getSameThemeOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        themeId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse>

    fun findByTheme(
        memberId: Long,
        theme: List<String> = emptyList(),
        sortType: SortType = SortType.NEWEST,
        isAdult: Boolean = false,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20,
        isFree: Boolean = false
    ): List<GetAudioContentMainItem>

    fun findAlarmContentByTheme(
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean = false,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun totalAlarmCountByTheme(
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean = false,
        contentType: ContentType
    ): Int

    fun totalCountByTheme(
        memberId: Long,
        theme: List<String> = emptyList(),
        isAdult: Boolean = false,
        contentType: ContentType
    ): Int

    fun findByThemeFor2Weeks(
        isFree: Boolean = false,
        cloudfrontHost: String,
        memberId: Long,
        theme: List<String> = emptyList(),
        isAdult: Boolean = false,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun totalCountNewContentFor2Weeks(
        isFree: Boolean = false,
        theme: List<String> = emptyList(),
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): Int

    fun getNewContentUploadCreatorList(
        cloudfrontHost: String,
        isAdult: Boolean = false
    ): List<ContentCreatorResponse>

    fun getAudioContentMainBannerList(isAdult: Boolean): List<AudioContentBanner>
    fun getAudioContentCurations(isAdult: Boolean): List<AudioContentCuration>
    fun findAudioContentByCurationId(
        curationId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetAudioContentMainItem>

    fun findAudioContentByCurationIdV2(
        curationId: Long,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetAudioContentMainItem>

    fun getAudioContentRanking(
        cloudfrontHost: String,
        isAdult: Boolean,
        contentType: ContentType,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long = 0,
        limit: Long = 12,
        sortType: String = "매출"
    ): List<GetAudioContentRankingItem>

    fun getAudioContentCurationList(isAdult: Boolean, offset: Long, limit: Long): List<AudioContentCuration>

    fun getNotReleaseContent(): List<AudioContent>

    fun isContentCreator(contentId: Long, memberId: Long): Boolean

    fun fetchContentForPlaylist(contentIdList: List<Long>): List<AudioContentPlaylistContent>
    fun getCoverImageById(id: Long): String?
    fun findPreviousContent(seriesId: Long, title: String, isAdult: Boolean): OtherContentResponse?
    fun findNextContent(seriesId: Long, title: String, isAdult: Boolean): OtherContentResponse?

    fun findSeriesIdByContentId(contentId: Long, isAdult: Boolean): Long?
}

@Repository
class AudioContentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AudioContentQueryRepository {
    override fun findByIdAndActive(contentId: Long): AudioContent? {
        return queryFactory
            .selectFrom(audioContent)
            .where(
                audioContent.isActive.isTrue
                    .and(audioContent.id.eq(contentId))
            )
            .fetchOne()
    }

    override fun findByIdAndCreatorId(contentId: Long, creatorId: Long): AudioContent? {
        return queryFactory
            .selectFrom(audioContent)
            .where(
                audioContent.id.eq(contentId)
                    .and(audioContent.member.id.eq(creatorId))
            )
            .fetchOne()
    }

    override fun findByCreatorId(
        creatorId: Long,
        coverImageHost: String,
        isAdult: Boolean,
        contentType: ContentType,
        sortType: SortType,
        categoryId: Long,
        offset: Long,
        limit: Long
    ): List<GetAudioContentListItem> {
        val orderBy = when (sortType) {
            SortType.NEWEST -> audioContent.releaseDate.desc()
            SortType.PRICE_HIGH -> audioContent.price.desc()
            SortType.PRICE_LOW -> audioContent.price.asc()
        }

        var where = audioContent.member.id.eq(creatorId)
            .and(
                audioContent.isActive.isTrue
                    .and(audioContent.duration.isNotNull)
                    .or(audioContent.releaseDate.isNotNull.and(audioContent.duration.isNotNull))
            )

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        var select = queryFactory
            .select(
                QGetAudioContentListItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(coverImageHost),
                    audioContent.title,
                    audioContent.price,
                    audioContent.theme.theme,
                    audioContent.duration,
                    Expressions.constant(0),
                    Expressions.constant(0),
                    pinContent.id.isNotNull,
                    audioContent.isAdult,
                    audioContent.releaseDate.gt(LocalDateTime.now()),
                    Expressions.asBoolean(false),
                    Expressions.asBoolean(false),
                    audioContent.remaining.loe(0)
                )
            )
            .from(audioContent)
            .leftJoin(pinContent)
            .on(audioContent.id.eq(pinContent.content.id).and(pinContent.isActive.ne(false)))

        select = if (categoryId > 0) {
            select
                .leftJoin(categoryContent)
                .on(audioContent.id.eq(categoryContent.content.id).and(categoryContent.isActive.ne(false)))
                .where(where.and(categoryContent.category.id.eq(categoryId)))
        } else {
            select.where(where)
        }

        select = select
            .offset(offset)
            .limit(limit)

        select = if (categoryId > 0) {
            select
                .orderBy(
                    pinContent.isActive.desc(),
                    pinContent.updatedAt.desc(),
                    orderBy,
                    categoryContent.updatedAt.desc(),
                    audioContent.id.desc()
                )
        } else {
            select
                .orderBy(
                    pinContent.isActive.desc(),
                    pinContent.updatedAt.desc(),
                    orderBy,
                    audioContent.updatedAt.desc(),
                    audioContent.id.desc()
                )
        }

        return select.fetch()
    }

    override fun findTotalCountByCreatorId(
        creatorId: Long,
        isAdult: Boolean,
        categoryId: Long,
        contentType: ContentType
    ): Int {
        var where = audioContent.member.id.eq(creatorId)
            .and(
                audioContent.isActive.isTrue
                    .and(audioContent.duration.isNotNull)
                    .or(audioContent.releaseDate.isNotNull.and(audioContent.duration.isNotNull))
            )

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        var select = queryFactory
            .select(audioContent.id)
            .from(audioContent)

        select = if (categoryId > 0) {
            select
                .leftJoin(categoryContent)
                .on(audioContent.id.eq(categoryContent.content.id).and(categoryContent.isActive.ne(false)))
                .where(where.and(categoryContent.category.id.eq(categoryId)))
        } else {
            select.where(where)
        }

        return select.fetch().size
    }

    override fun getCreatorOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        creatorId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse> {
        var where = audioContent.id.ne(contentId)
            .and(audioContent.member.id.eq(creatorId))
            .and(audioContent.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$cloudfrontHost/")
                )
            )
            .from(audioContent)
            .where(where)
            .offset(0)
            .limit(10)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .fetch()
    }

    override fun getSameThemeOtherContentList(
        cloudfrontHost: String,
        contentId: Long,
        themeId: Long,
        isAdult: Boolean
    ): List<OtherContentResponse> {
        var where = audioContent.id.ne(contentId)
            .and(audioContent.theme.id.eq(themeId))
            .and(audioContent.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$cloudfrontHost/")
                )
            )
            .from(audioContent)
            .where(where)
            .offset(0)
            .limit(10)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .fetch()
    }

    override fun findByTheme(
        memberId: Long,
        theme: List<String>,
        sortType: SortType,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long,
        isFree: Boolean
    ): List<GetAudioContentMainItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val orderBy = when (sortType) {
            SortType.NEWEST -> listOf(audioContent.releaseDate.desc(), audioContent.id.desc())
            SortType.PRICE_HIGH -> listOf(
                audioContent.price.desc(),
                audioContent.releaseDate.desc(),
                audioContent.id.desc()
            )

            SortType.PRICE_LOW -> listOf(
                audioContent.price.asc(),
                audioContent.releaseDate.asc(),
                audioContent.id.asc()
            )
        }

        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        where = if (isFree) {
            where.and(audioContent.price.loe(0))
        } else {
            where.and(audioContent.price.gt(0))
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(imageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(*orderBy.toTypedArray())
            .fetch()
    }

    override fun findAlarmContentByTheme(
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        val orderBy = listOf(audioContent.releaseDate.desc(), audioContent.id.desc())

        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(imageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(*orderBy.toTypedArray())
            .fetch()
    }

    override fun totalAlarmCountByTheme(
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean,
        contentType: ContentType
    ): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .fetch()
            .size
    }

    override fun totalCountByTheme(
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean,
        contentType: ContentType
    ): Int {
        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .fetch()
            .size
    }

    override fun totalCountNewContentFor2Weeks(
        isFree: Boolean,
        theme: List<String>,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): Int {
        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(audioContent.releaseDate.goe(LocalDateTime.now().minusWeeks(2)))
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        if (isFree) {
            where = where.and(audioContent.price.loe(0))
        }

        return queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .fetch()
            .size
    }

    override fun findByThemeFor2Weeks(
        isFree: Boolean,
        cloudfrontHost: String,
        memberId: Long,
        theme: List<String>,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.duration.isNotNull)
            .and(audioContent.releaseDate.goe(LocalDateTime.now().minusWeeks(2)))
            .and(
                audioContent.releaseDate.isNull
                    .or(audioContent.releaseDate.loe(LocalDateTime.now()))
                    .or(audioContent.member.id.eq(memberId))
            )

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        if (theme.isNotEmpty()) {
            where = where.and(audioContentTheme.theme.`in`(theme))
        }

        if (isFree) {
            where = where.and(audioContent.price.loe(0))
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(cloudfrontHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContent.releaseDate.desc(), audioContent.id.desc())
            .fetch()
    }

    override fun getNewContentUploadCreatorList(
        cloudfrontHost: String,
        isAdult: Boolean
    ): List<ContentCreatorResponse> {
        var where = audioContent.releaseDate.after(LocalDateTime.now().minusWeeks(2))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.duration.isNotNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(member)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .where(where)
            .groupBy(member.id)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .limit(20)
            .fetch()
            .map {
                ContentCreatorResponse(
                    it.id!!,
                    it.nickname,
                    creatorProfileImageUrl = if (it.profileImage != null) {
                        "$cloudfrontHost/${it.profileImage}"
                    } else {
                        "$cloudfrontHost/profile/default-profile.png"
                    }
                )
            }
    }

    override fun getAudioContentMainBannerList(isAdult: Boolean): List<AudioContentBanner> {
        var where = audioContentBanner.isActive.isTrue
            .and(audioContentBanner.tab.isNull)

        if (!isAdult) {
            where = where.and(audioContentBanner.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentBanner)
            .leftJoin(audioContentBanner.tab, audioContentMainTab)
            .leftJoin(audioContentBanner.event, event)
            .leftJoin(audioContentBanner.creator, member)
            .where(where)
            .orderBy(audioContentBanner.orders.asc())
            .fetch()
    }

    override fun getAudioContentCurations(isAdult: Boolean): List<AudioContentCuration> {
        var where = audioContentCuration.isActive.isTrue

        if (!isAdult) {
            where = where.and(audioContentCuration.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentCuration)
            .where(where)
            .orderBy(audioContentCuration.orders.asc())
            .fetch()
    }

    override fun findAudioContentByCurationId(
        curationId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetAudioContentMainItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isNotNull)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.curation.id.eq(curationId))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage
                        .prepend("/")
                        .prepend(imageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(audioContent.curation, audioContentCuration)
            .where(where)
            .orderBy(audioContent.id.desc())
            .fetch()
    }

    override fun findAudioContentByCurationIdV2(
        curationId: Long,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<GetAudioContentMainItem> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = audioContentCuration.isActive.isTrue
            .and(audioContentCurationItem.isActive.isTrue)
            .and(audioContent.isActive.isTrue)
            .and(audioContent.member.isNotNull)
            .and(audioContent.duration.isNotNull)
            .and(audioContent.member.isActive.isTrue)
            .and(audioContentCuration.id.eq(curationId))
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QGetAudioContentMainItem(
                    audioContent.id,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.title,
                    member.id,
                    member.profileImage.prepend("/").prepend(imageHost),
                    member.nickname,
                    audioContent.price,
                    audioContent.duration
                )
            )
            .from(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.content, audioContent)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContent.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getAudioContentRanking(
        cloudfrontHost: String,
        isAdult: Boolean,
        contentType: ContentType,
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long,
        limit: Long,
        sortType: String
    ): List<GetAudioContentRankingItem> {
        var where = audioContent.isActive.isTrue
            .and(audioContent.member.isActive.isTrue)
            .and(audioContent.member.isNotNull)
            .and(audioContent.member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContentTheme.isActive.isTrue)
            .and(audioContent.limited.isNull)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        var select = queryFactory
            .select(
                QGetAudioContentRankingItem(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("/").prepend(cloudfrontHost),
                    audioContentTheme.theme,
                    audioContent.price,
                    audioContent.duration,
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )

        select = when (sortType) {
            "후원" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContentComment.audioContent.member, member)
                    .innerJoin(audioContentComment.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.donationCan.gt(0))
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(audioContentComment.donationCan.sum().desc(), audioContent.createdAt.asc())
            }

            "댓글" -> {
                select
                    .from(audioContentComment)
                    .innerJoin(audioContentComment.audioContent, audioContent)
                    .innerJoin(audioContentComment.audioContent.member, member)
                    .innerJoin(audioContentComment.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentComment.isActive.isTrue)
                            .and(audioContentComment.createdAt.goe(startDate))
                            .and(audioContentComment.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentComment.audioContent.id)
                    .orderBy(audioContentComment.id.count().desc(), audioContent.createdAt.asc())
            }

            "좋아요" -> {
                select
                    .from(audioContentLike)
                    .innerJoin(audioContentLike.audioContent, audioContent)
                    .innerJoin(audioContentLike.audioContent.member, member)
                    .innerJoin(audioContentLike.audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(audioContentLike.isActive.isTrue)
                            .and(audioContentLike.createdAt.goe(startDate))
                            .and(audioContentLike.createdAt.lt(endDate))
                    )
                    .groupBy(audioContentLike.audioContent.id)
                    .orderBy(audioContentLike.id.count().desc(), audioContent.createdAt.asc())
            }

            else -> {
                select
                    .from(order)
                    .innerJoin(order.audioContent, audioContent)
                    .innerJoin(audioContent.member, member)
                    .innerJoin(audioContent.theme, audioContentTheme)
                    .where(
                        where
                            .and(order.isActive.isTrue)
                            .and(order.createdAt.goe(startDate))
                            .and(order.createdAt.lt(endDate))
                    )
                    .groupBy(audioContent.id)
                    .orderBy(order.can.sum().desc(), audioContent.createdAt.asc())
            }
        }

        return select
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getAudioContentCurationList(isAdult: Boolean, offset: Long, limit: Long): List<AudioContentCuration> {
        var where = audioContentCuration.isActive.isTrue

        if (!isAdult) {
            where = where.and(audioContentCuration.isAdult.isFalse)
        }

        return queryFactory
            .selectFrom(audioContentCuration)
            .leftJoin(audioContentCuration.tab, audioContentMainTab)
            .where(where, audioContentMainTab.id.isNull)
            .offset(offset)
            .limit(limit)
            .orderBy(audioContentCuration.orders.asc())
            .fetch()
    }

    override fun getNotReleaseContent(): List<AudioContent> {
        val where = audioContent.isActive.isFalse
            .and(audioContent.releaseDate.isNotNull)
            .and(audioContent.releaseDate.loe(LocalDateTime.now()))
            .and(audioContent.duration.isNotNull)

        return queryFactory
            .selectFrom(audioContent)
            .where(where)
            .fetch()
    }

    override fun isContentCreator(contentId: Long, memberId: Long): Boolean {
        val foundedContentId = queryFactory
            .select(audioContent.id)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                audioContent.isActive.isTrue
                    .and(audioContent.id.eq(contentId))
                    .and(audioContent.member.id.eq(memberId))
            )
            .fetchOne()

        return foundedContentId != null && foundedContentId == contentId
    }

    override fun fetchContentForPlaylist(contentIdList: List<Long>): List<AudioContentPlaylistContent> {
        return queryFactory
            .select(
                QAudioContentPlaylistContent(
                    audioContent.id,
                    audioContent.title,
                    audioContentTheme.theme,
                    audioContent.coverImage.prepend("/").prepend(imageHost),
                    audioContent.duration,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(audioContent.id.`in`(contentIdList))
            .fetch()
    }

    override fun getCoverImageById(id: Long): String? {
        return queryFactory
            .select(audioContent.coverImage.prepend("/").prepend(imageHost))
            .from(audioContent)
            .where(audioContent.id.eq(id))
            .fetchFirst()
    }

    override fun findPreviousContent(seriesId: Long, title: String, isAdult: Boolean): OtherContentResponse? {
        var where = series.isActive.isTrue
            .and(series.id.eq(seriesId))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.title.lt(title))
            .and(audioContent.limited.isNull)
            .and(audioContent.releaseDate.loe(LocalDateTime.now()))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$imageHost/")
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .where(where)
            .orderBy(audioContent.title.desc())
            .limit(1)
            .fetchFirst()
    }

    override fun findNextContent(seriesId: Long, title: String, isAdult: Boolean): OtherContentResponse? {
        var where = series.isActive.isTrue
            .and(series.id.eq(seriesId))
            .and(audioContent.isActive.isTrue)
            .and(audioContent.title.gt(title))
            .and(audioContent.limited.isNull)
            .and(audioContent.releaseDate.loe(LocalDateTime.now()))

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        }

        return queryFactory
            .select(
                QOtherContentResponse(
                    audioContent.id,
                    audioContent.title,
                    audioContent.coverImage.prepend("$imageHost/")
                )
            )
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .where(where)
            .orderBy(audioContent.title.asc())
            .limit(1)
            .fetchFirst()
    }

    override fun findSeriesIdByContentId(contentId: Long, isAdult: Boolean): Long? {
        var where = series.isActive.isTrue
            .and(audioContent.isActive.isTrue)
            .and(audioContent.id.eq(contentId))
            .and(member.role.eq(MemberRole.CREATOR))

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        }

        return queryFactory
            .select(series.id)
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .where(where)
            .orderBy(seriesContent.id.asc())
            .limit(1)
            .fetchFirst()
    }
}
