package kr.co.vividnext.sodalive.content.main.tab

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.tab.QRecommendSeries.recommendSeries
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class RecommendSeriesRepository(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getNewSeriesList(isAdult: Boolean, contentType: ContentType): List<GetRecommendSeriesListResponse> {
        var where = recommendSeries.isActive.isTrue
            .and(recommendSeries.isFree.isFalse)
            .and(series.isActive.isTrue)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QGetRecommendSeriesListResponse(
                    series.id,
                    series.title,
                    recommendSeries.imagePath.prepend("/").prepend(imageHost),
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(recommendSeries)
            .innerJoin(recommendSeries.series, series)
            .innerJoin(series.member, member)
            .where(where)
            .orderBy(recommendSeries.orders.asc())
            .fetch()
    }

    fun getRecommendSeriesList(isAdult: Boolean, contentType: ContentType): List<GetRecommendSeriesListResponse> {
        var where = recommendSeries.isActive.isTrue
            .and(recommendSeries.isFree.isTrue)
            .and(series.isActive.isTrue)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(
                QGetRecommendSeriesListResponse(
                    series.id,
                    series.title,
                    recommendSeries.imagePath.prepend("/").prepend(imageHost),
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(imageHost)
                )
            )
            .from(recommendSeries)
            .innerJoin(recommendSeries.series, series)
            .innerJoin(series.member, member)
            .where(where)
            .orderBy(recommendSeries.orders.asc())
            .fetch()
    }
}
