package kr.co.vividnext.sodalive.content.main.tab.replay

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/v2/audio-content/main/replay")
class AudioContentMainTabLiveReplayController(private val service: AudioContentMainTabLiveReplayService) {
    @GetMapping
    fun fetchContentMainTabLiveReplay(
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.fetchData(
                isAdultContentVisible = isAdultContentVisible ?: true,
                contentType = contentType ?: ContentType.ALL,
                member
            )
        )
    }

    @GetMapping("/popular-content-by-creator")
    fun getPopularContentByCreator(
        @RequestParam creatorId: Long,
        @RequestParam("isAdultContentVisible", required = false) isAdultContentVisible: Boolean? = null,
        @RequestParam("contentType", required = false) contentType: ContentType? = null,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getPopularContentByCreator(
                creatorId = creatorId,
                isAdult = member.auth != null && (isAdultContentVisible ?: true),
                contentType = contentType ?: ContentType.ALL
            )
        )
    }
}
