package kr.co.vividnext.sodalive.content.theme

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.theme.QAudioContentTheme.audioContentTheme
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Repository

@Repository
class AudioContentThemeQueryRepository(
    private val queryFactory: JPAQueryFactory,
    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    fun getActiveThemes(): List<GetAudioContentThemeResponse> {
        return queryFactory
            .select(
                QGetAudioContentThemeResponse(
                    audioContentTheme.id,
                    audioContentTheme.theme,
                    audioContentTheme.image.prepend("/").prepend(cloudFrontHost)
                )
            )
            .from(audioContentTheme)
            .where(audioContentTheme.isActive.isTrue)
            .orderBy(audioContentTheme.orders.asc())
            .fetch()
    }

    fun getActiveThemeOfContent(
        isAdult: Boolean = false,
        isFree: Boolean = false,
        contentType: ContentType
    ): List<String> {
        var where = audioContent.isActive.isTrue
            .and(audioContentTheme.isActive.isTrue)

        if (!isAdult) {
            where = where.and(audioContent.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    audioContent.member.isNull.or(
                        audioContent.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        where = if (isFree) {
            where.and(audioContent.price.loe(0))
        } else {
            where.and(audioContent.price.gt(0))
        }

        return queryFactory
            .select(audioContentTheme.theme)
            .from(audioContent)
            .innerJoin(audioContent.member, member)
            .innerJoin(audioContent.theme, audioContentTheme)
            .where(where)
            .groupBy(audioContentTheme.id)
            .orderBy(audioContentTheme.orders.asc())
            .fetch()
    }

    fun findThemeByIdAndActive(id: Long): AudioContentTheme? {
        return queryFactory
            .selectFrom(audioContentTheme)
            .where(
                audioContentTheme.id.eq(id)
                    .and(audioContentTheme.isActive.isTrue)
            )
            .fetchFirst()
    }
}
