package kr.co.vividnext.sodalive.live.room.info

import kr.co.vividnext.sodalive.live.room.donation.LiveRoomDonationMessage
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.annotation.Id
import org.springframework.data.redis.core.RedisHash

@RedisHash("live_room_info")
data class LiveRoomInfo(
    @Id
    val roomId: Long,
    var speakerList: List<LiveRoomMember> = mutableListOf(),
    var listenerList: List<LiveRoomMember> = mutableListOf(),
    var managerList: List<LiveRoomMember> = mutableListOf(),
    var donationMessageList: List<LiveRoomDonationMessage> = mutableListOf()
) {
    var speakerCount = 0
        private set

    var listenerCount = 0
        private set

    var managerCount = 0
        private set

    fun addSpeaker(member: Member, cloudFrontHost: String) {
        val liveRoomMember = LiveRoomMember(member, cloudFrontHost)
        liveRoomMember.role = LiveRoomMemberRole.SPEAKER

        val speakerSet = speakerList.toMutableSet()
        speakerSet.add(liveRoomMember)
        speakerList = speakerSet.toList()

        setSpeakerCount()
    }

    fun removeSpeaker(member: Member) {
        (speakerList as MutableList).removeIf { it.id == member.id!! }
        setSpeakerCount()
    }

    private fun setSpeakerCount() {
        speakerCount = speakerList.size
    }

    fun addListener(member: Member, cloudFrontHost: String) {
        val liveRoomMember = LiveRoomMember(member, cloudFrontHost)
        liveRoomMember.role = LiveRoomMemberRole.LISTENER

        val listenerSet = listenerList.toMutableSet()
        listenerSet.add(liveRoomMember)
        listenerList = listenerSet.toList()

        setListenerCount()
    }

    fun removeListener(member: Member) {
        (listenerList as MutableList).removeIf { it.id == member.id!! }
        setListenerCount()
    }

    private fun setListenerCount() {
        listenerCount = listenerList.size
    }

    fun addManager(member: Member, cloudFrontHost: String) {
        val liveRoomMember = LiveRoomMember(member, cloudFrontHost)
        liveRoomMember.role = LiveRoomMemberRole.MANAGER

        val managerSet = managerList.toMutableSet()
        managerSet.add(liveRoomMember)
        managerList = managerSet.toList()

        setManagerCount()
    }

    fun removeManager(member: Member) {
        (managerList as MutableList).removeIf { it.id == member.id!! }
        setManagerCount()
    }

    private fun setManagerCount() {
        managerCount = managerList.size
    }

    fun addDonationMessage(memberId: Long, nickname: String, isSecret: Boolean, can: Int, donationMessage: String) {
        val donationMessageSet = donationMessageList.toMutableSet()
        donationMessageSet.add(
            LiveRoomDonationMessage(
                memberId = memberId,
                nickname = nickname,
                isSecret = isSecret,
                canMessage = if (isSecret) {
                    "${can}캔으로 비밀미션을 보냈습니다."
                } else {
                    "${can}캔을 후원하셨습니다."
                },
                donationMessage = donationMessage
            )
        )
        donationMessageList = donationMessageSet.toList()
    }

    fun addRouletteMessage(memberId: Long, nickname: String, donationMessage: String) {
        val donationMessageSet = donationMessageList.toMutableSet()
        donationMessageSet.add(
            LiveRoomDonationMessage(
                memberId = memberId,
                nickname = nickname,
                isSecret = false,
                canMessage = "",
                donationMessage = donationMessage
            )
        )
        donationMessageList = donationMessageSet.toList()
    }

    fun removeDonationMessage(uuid: String) {
        (donationMessageList as MutableList).removeIf { it.uuid == uuid }
    }
}
