package kr.co.vividnext.sodalive.live.room.visit

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.live.room.visit.QLiveRoomVisit.liveRoomVisit
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface LiveRoomVisitRepository : JpaRepository<LiveRoomVisit, Long>, LiveRoomVisitQueryRepository

interface LiveRoomVisitQueryRepository {
    fun findByRoomIdAndMemberId(roomId: Long, memberId: Long): LiveRoomVisit?
    fun findFirstByMemberIdOrderByUpdatedAtDesc(memberId: Long): LiveRoomVisit?
    fun getRecentVisitRoomUsers(roomId: Long, member: Member): List<Member>
}

@Repository
class LiveRoomVisitQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : LiveRoomVisitQueryRepository {
    override fun findByRoomIdAndMemberId(roomId: Long, memberId: Long): LiveRoomVisit? {
        return queryFactory
            .selectFrom(liveRoomVisit)
            .innerJoin(liveRoomVisit.room, liveRoom)
            .innerJoin(liveRoomVisit.member, member)
            .where(
                liveRoom.id.eq(roomId)
                    .and(member.id.eq(memberId))
            )
            .orderBy(liveRoomVisit.id.desc())
            .fetchFirst()
    }

    override fun findFirstByMemberIdOrderByUpdatedAtDesc(memberId: Long): LiveRoomVisit? {
        return queryFactory
            .selectFrom(liveRoomVisit)
            .innerJoin(liveRoomVisit.room, liveRoom)
            .innerJoin(liveRoomVisit.member, member)
            .where(member.id.eq(memberId))
            .orderBy(liveRoomVisit.updatedAt.desc())
            .fetchFirst()
    }

    override fun getRecentVisitRoomUsers(roomId: Long, member: Member): List<Member> {
        var where = liveRoomVisit.room.id.eq(roomId)
            .and(liveRoomVisit.member.isActive.isTrue)
            .and(liveRoomVisit.member.id.ne(member.id))
            .and(liveRoomVisit.member.role.ne(MemberRole.ADMIN))
            .and(liveRoomVisit.member.role.ne(MemberRole.AGENT))
            .and(liveRoomVisit.member.role.ne(MemberRole.BOT))

        if (member.role == MemberRole.USER) {
            where = where
                .and(liveRoomVisit.member.role.ne(MemberRole.USER))
        }

        return queryFactory
            .select(QMember.member)
            .from(liveRoomVisit)
            .innerJoin(liveRoomVisit.member, QMember.member)
            .innerJoin(liveRoomVisit.room, liveRoom)
            .where(where)
            .fetch()
    }
}
