package kr.co.vividnext.sodalive.member.auth

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberService
import kr.co.vividnext.sodalive.useraction.ActionType
import kr.co.vividnext.sodalive.useraction.UserActionService
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/auth")
class AuthController(
    private val service: AuthService,
    private val memberService: MemberService,
    private val userActionService: UserActionService
) {
    @PostMapping
    fun authVerify(
        @RequestBody request: AuthVerifyRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        val authenticateData = service.certificate(request, memberId = member.id!!)

        if (service.isBlockAuth(authenticateData)) {
            service.signOut(member.id!!)
            throw SodaException("운영정책을 위반하여 이용을 제한합니다.")
        }

        val authResponse = service.authenticate(authenticateData, member.id!!)

        try {
            val memberId = member.id!!
            val pushTokenList = memberService.getPushTokenList(recipient = memberId)
            userActionService.recordAction(
                memberId = member.id!!,
                isAuth = true,
                actionType = ActionType.USER_AUTHENTICATION,
                pushTokenList = pushTokenList
            )
        } catch (_: Exception) {
        }

        ApiResponse.ok(authResponse)
    }
}
