package kr.co.vividnext.sodalive.point

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.QMemberPoint.memberPoint
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface MemberPointRepository : JpaRepository<MemberPoint, Long>, MemberPointQueryRepository

interface MemberPointQueryRepository {
    fun findByMemberIdAndExpiresAtAfterOrderByExpiresAtAsc(memberId: Long, expiresAt: LocalDateTime): List<MemberPoint>
}

class MemberPointQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : MemberPointQueryRepository {
    override fun findByMemberIdAndExpiresAtAfterOrderByExpiresAtAsc(
        memberId: Long,
        expiresAt: LocalDateTime
    ): List<MemberPoint> {
        return queryFactory
            .selectFrom(memberPoint)
            .where(
                memberPoint.memberId.eq(memberId),
                memberPoint.expiresAt.goe(expiresAt)
            )
            .fetch()
    }
}
