package kr.co.vividnext.sodalive.point

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.point.QPointRewardPolicy.pointRewardPolicy
import kr.co.vividnext.sodalive.useraction.ActionType
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface PointRewardPolicyRepository : JpaRepository<PointRewardPolicy, Long>, PointRewardPolicyQueryRepository

interface PointRewardPolicyQueryRepository {
    fun findByActionTypeAndIsActiveTrue(actionType: ActionType, nowDateTime: LocalDateTime): PointRewardPolicy?
}

class PointRewardPolicyQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : PointRewardPolicyQueryRepository {
    override fun findByActionTypeAndIsActiveTrue(
        actionType: ActionType,
        nowDateTime: LocalDateTime
    ): PointRewardPolicy? {
        return queryFactory
            .selectFrom(pointRewardPolicy)
            .where(
                pointRewardPolicy.isActive.isTrue
                    .and(pointRewardPolicy.actionType.eq(actionType))
                    .and(pointRewardPolicy.startDate.loe(nowDateTime))
                    .and(
                        pointRewardPolicy.endDate.goe(nowDateTime)
                            .or(pointRewardPolicy.endDate.isNull)
                    )
            )
            .orderBy(pointRewardPolicy.endDate.asc())
            .fetchFirst()
    }
}
